package sun.swing;

import java.util.Map;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeModel;
import javax.swing.event.TreeModelEvent;
import sun.awt.SunToolkit;
import javax.swing.table.TableColumnModel;
import javax.swing.text.DefaultCaret;
import java.util.concurrent.FutureTask;
import java.util.concurrent.Future;
import java.util.concurrent.Callable;
import java.awt.Container;
import javax.swing.UIManager;
import java.util.Locale;
import sun.java2d.SunGraphicsEnvironment;
import java.io.InputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.BufferedInputStream;
import javax.swing.UIDefaults;
import sun.awt.AppContext;
import java.lang.reflect.Modifier;
import java.awt.EventQueue;
import java.awt.event.KeyEvent;
import java.security.AccessController;
import java.security.PrivilegedAction;
import java.awt.event.InputEvent;
import java.awt.AWTEvent;
import sun.security.util.SecurityConstants;
import java.awt.GraphicsEnvironment;
import javax.swing.text.DefaultHighlighter;
import javax.swing.text.JTextComponent;
import javax.swing.text.Highlighter;
import java.awt.PrintGraphics;
import java.awt.print.PrinterGraphics;
import sun.font.FontDesignMetrics;
import sun.print.ProxyPrintGraphics;
import java.util.HashMap;
import java.text.AttributedCharacterIterator;
import javax.swing.SwingUtilities;
import java.awt.event.MouseEvent;
import javax.swing.ListSelectionModel;
import javax.swing.JTable;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Point;
import javax.swing.JList;
import java.awt.Rectangle;
import java.awt.font.TextHitInfo;
import java.awt.Color;
import java.awt.font.TextLayout;
import java.awt.RenderingHints;
import java.awt.Graphics2D;
import java.awt.font.LineBreakMeasurer;
import java.text.BreakIterator;
import java.text.AttributedString;
import java.awt.font.TextAttribute;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Font;
import java.awt.FontMetrics;
import javax.swing.JComponent;
import sun.font.FontUtilities;
import java.lang.reflect.Field;
import java.awt.font.FontRenderContext;

public class SwingUtilities2
{
    public static final Object LAF_STATE_KEY;
    public static final Object MENU_SELECTION_MANAGER_LISTENER_KEY;
    private static SwingUtilities2.LSBCacheEntry[] fontCache;
    private static final int CACHE_SIZE = 6;
    private static int nextIndex;
    private static SwingUtilities2.LSBCacheEntry searchKey;
    private static final int MIN_CHAR_INDEX = 87;
    private static final int MAX_CHAR_INDEX = 88;
    public static final FontRenderContext DEFAULT_FRC;
    public static final Object AA_TEXT_PROPERTY_KEY;
    public static final String IMPLIED_CR = "CR";
    private static final StringBuilder SKIP_CLICK_COUNT;
    public static final Object COMPONENT_UI_PROPERTY_KEY;
    public static final StringUIClientPropertyKey BASICMENUITEMUI_MAX_TEXT_OFFSET;
    private static Field inputEvent_CanAccessSystemClipboard_Field;
    private static final String UntrustedClipboardAccess = "UNTRUSTED_CLIPBOARD_ACCESS_KEY";
    private static final int CHAR_BUFFER_SIZE = 100;
    private static final Object charsBufferLock;
    private static char[] charsBuffer;
    
    public SwingUtilities2() {
    }
    
    private static int syncCharsBuffer(String s) {
        int length = s.length();
        if (SwingUtilities2.charsBuffer == null || SwingUtilities2.charsBuffer.length < length) {
            SwingUtilities2.charsBuffer = s.toCharArray();
        }
        else {
            s.getChars(0, length, SwingUtilities2.charsBuffer, 0);
        }
        return length;
    }
    
    public static final boolean isComplexLayout(char[] chs, int start, int limit) {
        return FontUtilities.isComplexText(chs, start, limit);
    }
    
    public static SwingUtilities2.AATextInfo drawTextAntialiased(JComponent component) {
        if (component != null) {
            return (SwingUtilities2.AATextInfo)component.getClientProperty(SwingUtilities2.AA_TEXT_PROPERTY_KEY);
        }
        return null;
    }
    
    public static int getLeftSideBearing(JComponent component, FontMetrics fontMetrics, String s) {
        if (s == null || s.length() == 0) {
            return 0;
        }
        return getLeftSideBearing(component, fontMetrics, s.charAt(0));
    }
    
    public static int getLeftSideBearing(JComponent component, FontMetrics fontMetrics, char c) {
        if (c < 'X' && c >= 'W') {
            FontRenderContext fontRenderContext = getFontRenderContext(component, fontMetrics);
            Font font = fontMetrics.getFont();
            synchronized (SwingUtilities2.class) {
                SwingUtilities2.LSBCacheEntry searchKey = null;
                if (SwingUtilities2.searchKey == null) {
                    SwingUtilities2.searchKey = new SwingUtilities2.LSBCacheEntry(fontRenderContext, font);
                }
                else {
                    SwingUtilities2.searchKey.reset(fontRenderContext, font);
                }
                SwingUtilities2.LSBCacheEntry[] fontCache = SwingUtilities2.fontCache;
                int length = fontCache.length;
                int i = 0;
                while (i < length) {
                    SwingUtilities2.LSBCacheEntry lsbCacheEntry = fontCache[i];
                    if (SwingUtilities2.searchKey.equals(lsbCacheEntry)) {
                        searchKey = lsbCacheEntry;
                        break;
                    }
                    ++i;
                }
                if (searchKey == null) {
                    searchKey = SwingUtilities2.searchKey;
                    SwingUtilities2.fontCache[SwingUtilities2.nextIndex] = SwingUtilities2.searchKey;
                    SwingUtilities2.searchKey = null;
                    SwingUtilities2.nextIndex = (SwingUtilities2.nextIndex + 1) % 6;
                }
                return searchKey.getLeftSideBearing(c);
            }
        }
        return 0;
    }
    
    public static FontMetrics getFontMetrics(JComponent component, Graphics graphics) {
        return getFontMetrics(component, graphics, graphics.getFont());
    }
    
    public static FontMetrics getFontMetrics(JComponent component, Graphics graphics, Font font) {
        if (component != null) {
            return component.getFontMetrics(font);
        }
        return Toolkit.getDefaultToolkit().getFontMetrics(font);
    }
    
    public static int stringWidth(JComponent component, FontMetrics fontMetrics, String str) {
        if (str == null || str.equals("")) {
            return 0;
        }
        int complexLayout = component != null && component.getClientProperty(TextAttribute.NUMERIC_SHAPING) != null;
        if (!(complexLayout == 0)) {
            synchronized (SwingUtilities2.charsBufferLock) {
                complexLayout = isComplexLayout(SwingUtilities2.charsBuffer, 0, syncCharsBuffer(str));
            }
        }
        if (!(complexLayout == 0)) {
            return (int)createTextLayout(component, str, fontMetrics.getFont(), fontMetrics.getFontRenderContext()).getAdvance();
        }
        return fontMetrics.stringWidth(str);
    }
    
    public static String clipStringIfNecessary(JComponent component, FontMetrics fontMetrics, String s, int n) {
        if (s == null || s.equals("")) {
            return "";
        }
        if (stringWidth(component, fontMetrics, s) > n) {
            return clipString(component, fontMetrics, s, n);
        }
        return s;
    }
    
    public static String clipString(JComponent component, FontMetrics fontMetrics, String s, int n) {
        String str = "...";
        n -= stringWidth(component, fontMetrics, str);
        if (n <= 0) {
            return str;
        }
        int complexLayout;
        synchronized (SwingUtilities2.charsBufferLock) {
            int syncCharsBuffer = syncCharsBuffer(s);
            complexLayout = isComplexLayout(SwingUtilities2.charsBuffer, 0, syncCharsBuffer);
            if (complexLayout == 0) {
                int n2 = 0;
                int i = 0;
                while (i < syncCharsBuffer) {
                    n2 += fontMetrics.charWidth(SwingUtilities2.charsBuffer[i]);
                    if (n2 > n) {
                        s = s.substring(0, i);
                        break;
                    }
                    ++i;
                }
            }
        }
        if (!(complexLayout == 0)) {
            AttributedString attributedString = new AttributedString(s);
            if (component != null) {
                attributedString.addAttribute(TextAttribute.NUMERIC_SHAPING, component.getClientProperty(TextAttribute.NUMERIC_SHAPING));
            }
            s = s.substring(0, new LineBreakMeasurer(attributedString.getIterator(), BreakIterator.getCharacterInstance(), getFontRenderContext(component, fontMetrics)).nextOffset((float)n));
        }
        return s + str;
    }
    
    public static void drawString(JComponent component, Graphics graphics, String s, int n, int n2) {
        if (s == null || s.length() <= 0) {
            return;
        }
        if (isPrinting(graphics)) {
            Graphics2D graphics2D = getGraphics2D(graphics);
            if (graphics2D != null) {
                String trimTrailingSpaces = trimTrailingSpaces(s);
                if (!trimTrailingSpaces.isEmpty()) {
                    float justificationWidth = (float)graphics2D.getFont().getStringBounds(trimTrailingSpaces, getFontRenderContext(component)).getWidth();
                    TextLayout textLayout = createTextLayout(component, s, graphics2D.getFont(), graphics2D.getFontRenderContext());
                    if ((float)stringWidth(component, graphics2D.getFontMetrics(), trimTrailingSpaces) > justificationWidth) {
                        textLayout = textLayout.getJustifiedLayout(justificationWidth);
                    }
                    Color color = graphics2D.getColor();
                    if ((color instanceof PrintColorUIResource)) {
                        graphics2D.setColor(((PrintColorUIResource)color).getPrintColor());
                    }
                    textLayout.draw(graphics2D, (float)n, (float)n2);
                    graphics2D.setColor(color);
                }
                return;
            }
        }
        if ((graphics instanceof Graphics2D)) {
            SwingUtilities2.AATextInfo drawTextAntialiased = drawTextAntialiased(component);
            Graphics2D graphics2D2 = (Graphics2D)graphics;
            int complexLayout = component != null && component.getClientProperty(TextAttribute.NUMERIC_SHAPING) != null;
            if (!(complexLayout == 0)) {
                synchronized (SwingUtilities2.charsBufferLock) {
                    complexLayout = isComplexLayout(SwingUtilities2.charsBuffer, 0, syncCharsBuffer(s));
                }
            }
            if (drawTextAntialiased != null) {
                Object renderingHint = null;
                Object renderingHint2 = graphics2D2.getRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING);
                if (drawTextAntialiased.aaHint != renderingHint2) {
                    graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, drawTextAntialiased.aaHint);
                }
                else {
                    renderingHint2 = null;
                }
                if (drawTextAntialiased.lcdContrastHint != null) {
                    renderingHint = graphics2D2.getRenderingHint(RenderingHints.KEY_TEXT_LCD_CONTRAST);
                    if (drawTextAntialiased.lcdContrastHint.equals(renderingHint)) {
                        renderingHint = null;
                    }
                    else {
                        graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_LCD_CONTRAST, drawTextAntialiased.lcdContrastHint);
                    }
                }
                if (!(complexLayout == 0)) {
                    createTextLayout(component, s, graphics2D2.getFont(), graphics2D2.getFontRenderContext()).draw(graphics2D2, (float)n, (float)n2);
                }
                else {
                    graphics.drawString(s, n, n2);
                }
                if (renderingHint2 != null) {
                    graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, renderingHint2);
                }
                if (renderingHint != null) {
                    graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_LCD_CONTRAST, renderingHint);
                }
                return;
            }
            else if (!(complexLayout == 0)) {
                createTextLayout(component, s, graphics2D2.getFont(), graphics2D2.getFontRenderContext()).draw(graphics2D2, (float)n, (float)n2);
                return;
            }
        }
        graphics.drawString(s, n, n2);
        return;
    }
    
    public static void drawStringUnderlineCharAt(JComponent component, Graphics graphics, String str, int n, int n2, int n3) {
        if (str == null || str.length() <= 0) {
            return;
        }
        drawString(component, graphics, str, n2, n3);
        int length = str.length();
        if (n >= 0 && n < length) {
            int n4 = 1;
            int n5 = 0;
            int n6 = 0;
            int n7 = isPrinting(graphics);
            if ((n7 = n7) == 0) {
                synchronized (SwingUtilities2.charsBufferLock) {
                    syncCharsBuffer(str);
                    n7 = isComplexLayout(SwingUtilities2.charsBuffer, 0, length);
                }
            }
            if (n7 == 0) {
                FontMetrics fontMetrics = graphics.getFontMetrics();
                n5 = n2 + stringWidth(component, fontMetrics, str.substring(0, n));
                n6 = fontMetrics.charWidth(str.charAt(n));
            }
            else {
                Graphics2D graphics2D = getGraphics2D(graphics);
                if (graphics2D != null) {
                    TextLayout textLayout = createTextLayout(component, str, graphics2D.getFont(), graphics2D.getFontRenderContext());
                    if (n7 != 0) {
                        float justificationWidth = (float)graphics2D.getFont().getStringBounds(str, getFontRenderContext(component)).getWidth();
                        if ((float)stringWidth(component, graphics2D.getFontMetrics(), str) > justificationWidth) {
                            textLayout = textLayout.getJustifiedLayout(justificationWidth);
                        }
                    }
                    Rectangle bounds = textLayout.getVisualHighlightShape(TextHitInfo.leading(n), TextHitInfo.trailing(n)).getBounds();
                    n5 = n2 + bounds.x;
                    n6 = bounds.width;
                }
            }
            graphics.fillRect(n5, n3 + 1, n6, n4);
        }
        return;
    }
    
    public static int loc2IndexFileList(JList list, Point location) {
        int locationToIndex = list.locationToIndex(location);
        if (locationToIndex != -1) {
            Object clientProperty = list.getClientProperty("List.isFileList");
            if (clientProperty instanceof Boolean && (boolean)clientProperty && !pointIsInActualBounds(list, locationToIndex, location)) {
                locationToIndex = -1;
            }
        }
        return locationToIndex;
    }
    
    private static boolean pointIsInActualBounds(JList list, int n, Point p3) {
        Component listCellRendererComponent = list.getCellRenderer().getListCellRendererComponent(list, list.getModel().getElementAt(n), n, false, false);
        Dimension preferredSize = listCellRendererComponent.getPreferredSize();
        Rectangle cellBounds = list.getCellBounds(n, n);
        if (!listCellRendererComponent.getComponentOrientation().isLeftToRight()) {
            cellBounds.x += cellBounds.width - preferredSize.width;
        }
        cellBounds.width = preferredSize.width;
        return cellBounds.contains(p3);
    }
    
    public static boolean pointOutsidePrefSize(JTable table, int row, int n, Point point) {
        if (table.convertColumnIndexToModel(n) != 0 || row == -1) {
            return true;
        }
        Dimension preferredSize = table.getCellRenderer(row, n).getTableCellRendererComponent(table, table.getValueAt(row, n), false, false, row, n).getPreferredSize();
        Rectangle cellRect = table.getCellRect(row, n, false);
        cellRect.width = preferredSize.width;
        cellRect.height = preferredSize.height;
        assert point.x >= cellRect.x && point.y >= cellRect.y;
        return point.x > cellRect.x + cellRect.width || point.y > cellRect.y + cellRect.height;
    }
    
    public static void setLeadAnchorWithoutSelection(ListSelectionModel listSelectionModel, int n, int anchorSelectionIndex) {
        if (anchorSelectionIndex == -1) {
            anchorSelectionIndex = n;
        }
        if (n == -1) {
            listSelectionModel.setAnchorSelectionIndex(-1);
            listSelectionModel.setLeadSelectionIndex(-1);
        }
        else {
            if (listSelectionModel.isSelectedIndex(n)) {
                listSelectionModel.addSelectionInterval(n, n);
            }
            else {
                listSelectionModel.removeSelectionInterval(n, n);
            }
            listSelectionModel.setAnchorSelectionIndex(anchorSelectionIndex);
        }
    }
    
    public static boolean shouldIgnore(MouseEvent anEvent, JComponent component) {
        return component == null || !component.isEnabled() || !SwingUtilities.isLeftMouseButton(anEvent) || anEvent.isConsumed();
    }
    
    public static void adjustFocus(JComponent component) {
        if (!component.hasFocus() && component.isRequestFocusEnabled()) {
            component.requestFocus();
        }
    }
    
    public static int drawChars(JComponent component, Graphics graphics, char[] array, int n, int n2, int n3, int n4) {
        if (n2 <= 0) {
            return n3;
        }
        int n5 = n3 + getFontMetrics(component, graphics).charsWidth(array, n, n2);
        if (isPrinting(graphics)) {
            Graphics2D graphics2D = getGraphics2D(graphics);
            if (graphics2D != null) {
                FontRenderContext fontRenderContext = graphics2D.getFontRenderContext();
                FontRenderContext fontRenderContext2 = getFontRenderContext(component);
                if (fontRenderContext2 != null && !isFontRenderContextPrintCompatible(fontRenderContext, fontRenderContext2)) {
                    String string = new String(array, n, n2);
                    TextLayout justifiedLayout = new TextLayout(string, graphics2D.getFont(), fontRenderContext);
                    String trimTrailingSpaces = trimTrailingSpaces(string);
                    if (!trimTrailingSpaces.isEmpty()) {
                        float justificationWidth = (float)graphics2D.getFont().getStringBounds(trimTrailingSpaces, fontRenderContext2).getWidth();
                        if ((float)stringWidth(component, graphics2D.getFontMetrics(), trimTrailingSpaces) > justificationWidth) {
                            justifiedLayout = justifiedLayout.getJustifiedLayout(justificationWidth);
                        }
                        Color color = graphics2D.getColor();
                        if ((color instanceof PrintColorUIResource)) {
                            graphics2D.setColor(((PrintColorUIResource)color).getPrintColor());
                        }
                        justifiedLayout.draw(graphics2D, (float)n3, (float)n4);
                        graphics2D.setColor(color);
                    }
                    return n5;
                }
            }
        }
        SwingUtilities2.AATextInfo drawTextAntialiased = drawTextAntialiased(component);
        if (drawTextAntialiased != null && graphics instanceof Graphics2D) {
            Graphics2D graphics2D2 = (Graphics2D)graphics;
            Object renderingHint = null;
            Object renderingHint2 = graphics2D2.getRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING);
            if (drawTextAntialiased.aaHint != null && drawTextAntialiased.aaHint != renderingHint2) {
                graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, drawTextAntialiased.aaHint);
            }
            else {
                renderingHint2 = null;
            }
            if (drawTextAntialiased.lcdContrastHint != null) {
                renderingHint = graphics2D2.getRenderingHint(RenderingHints.KEY_TEXT_LCD_CONTRAST);
                if (drawTextAntialiased.lcdContrastHint.equals(renderingHint)) {
                    renderingHint = null;
                }
                else {
                    graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_LCD_CONTRAST, drawTextAntialiased.lcdContrastHint);
                }
            }
            graphics.drawChars(array, n, n2, n3, n4);
            if (renderingHint2 != null) {
                graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, renderingHint2);
            }
            if (renderingHint != null) {
                graphics2D2.setRenderingHint(RenderingHints.KEY_TEXT_LCD_CONTRAST, renderingHint);
            }
        }
        else {
            graphics.drawChars(array, n, n2, n3, n4);
        }
        return n5;
    }
    
    public static float drawString(JComponent component, Graphics graphics, AttributedCharacterIterator text, int n, int n2) {
        int printing = isPrinting(graphics);
        Color color = graphics.getColor();
        if (printing && color instanceof PrintColorUIResource) {
            graphics.setColor(((PrintColorUIResource)color).getPrintColor());
        }
        Graphics2D graphics2D = getGraphics2D(graphics);
        float advance;
        if (graphics2D == null) {
            graphics.drawString(text, n, n2);
            advance = (float)n;
        }
        else {
            FontRenderContext frc;
            if (!(printing == 0)) {
                frc = getFontRenderContext(component);
                if (frc.isAntiAliased() || frc.usesFractionalMetrics()) {
                    frc = new FontRenderContext(frc.getTransform(), false, false);
                }
            }
            else if ((frc = getFRCProperty(component)) == null) {
                frc = graphics2D.getFontRenderContext();
            }
            Object justifiedLayout;
            if (!(printing == 0)) {
                FontRenderContext fontRenderContext = graphics2D.getFontRenderContext();
                if (!isFontRenderContextPrintCompatible(frc, fontRenderContext)) {
                    justifiedLayout = new TextLayout(text, fontRenderContext);
                    AttributedCharacterIterator trimmedTrailingSpacesIterator = getTrimmedTrailingSpacesIterator(text);
                    if (trimmedTrailingSpacesIterator != null) {
                        justifiedLayout = justifiedLayout.getJustifiedLayout(new TextLayout(trimmedTrailingSpacesIterator, frc).getAdvance());
                    }
                }
                else {
                    justifiedLayout = new TextLayout(text, frc);
                }
            }
            else {
                justifiedLayout = new TextLayout(text, frc);
            }
            justifiedLayout.draw(graphics2D, (float)n, (float)n2);
            advance = justifiedLayout.getAdvance();
        }
        if (!(printing == 0)) {
            graphics.setColor(color);
        }
        return advance;
    }
    
    public static void drawVLine(Graphics graphics, int n, int n2, int n3) {
        if (n3 < n2) {
            n3 = (n2 = n2);
        }
        graphics.fillRect(n, n2, 1, n3 - n2 + 1);
    }
    
    public static void drawHLine(Graphics graphics, int n, int n2, int n3) {
        if (n2 < n) {
            n2 = (n = n);
        }
        graphics.fillRect(n, n3, n2 - n + 1, 1);
    }
    
    public static void drawRect(Graphics graphics, int n, int n2, int n3, int n4) {
        if (n3 < 0 || n4 < 0) {
            return;
        }
        if (n4 == 0 || n3 == 0) {
            graphics.fillRect(n, n2, n3 + 1, n4 + 1);
        }
        else {
            graphics.fillRect(n, n2, n3, 1);
            graphics.fillRect(n + n3, n2, 1, n4);
            graphics.fillRect(n + 1, n2 + n4, n3, 1);
            graphics.fillRect(n, n2 + 1, 1, n4);
        }
        return;
    }
    
    private static TextLayout createTextLayout(JComponent component, String s, Font font, FontRenderContext fontRenderContext) {
        Object o = (component == null) ? null : component.getClientProperty(TextAttribute.NUMERIC_SHAPING);
        if (o == null) {
            return new TextLayout(s, font, fontRenderContext);
        }
        HashMap<TextAttribute, Object> attributes = new HashMap();
        attributes.put(TextAttribute.FONT, font);
        attributes.put(TextAttribute.NUMERIC_SHAPING, o);
        return new TextLayout(s, attributes, fontRenderContext);
    }
    
    private static boolean isFontRenderContextPrintCompatible(FontRenderContext fontRenderContext, FontRenderContext fontRenderContext2) {
        if (fontRenderContext == fontRenderContext2) {
            return true;
        }
        if (fontRenderContext == null || fontRenderContext2 == null) {
            return false;
        }
        if (fontRenderContext.getFractionalMetricsHint() != fontRenderContext2.getFractionalMetricsHint()) {
            return false;
        }
        if (!fontRenderContext.isTransformed() && !fontRenderContext2.isTransformed()) {
            return true;
        }
        double[] flatmatrix = new double[4];
        double[] flatmatrix2 = new double[4];
        fontRenderContext.getTransform().getMatrix(flatmatrix);
        fontRenderContext2.getTransform().getMatrix(flatmatrix2);
        return flatmatrix[0] == flatmatrix2[0] && flatmatrix[1] == flatmatrix2[1] && flatmatrix[2] == flatmatrix2[2] && flatmatrix[3] == flatmatrix2[3];
    }
    
    public static Graphics2D getGraphics2D(Graphics graphics) {
        if ((graphics instanceof Graphics2D)) {
            return (Graphics2D)graphics;
        }
        if ((graphics instanceof ProxyPrintGraphics)) {
            return (Graphics2D)(Graphics2D)((ProxyPrintGraphics)graphics).getGraphics();
        }
        return null;
    }
    
    public static FontRenderContext getFontRenderContext(Component component) {
        assert component != null;
        if (component == null) {
            return SwingUtilities2.DEFAULT_FRC;
        }
        return component.getFontMetrics(component.getFont()).getFontRenderContext();
    }
    
    private static FontRenderContext getFontRenderContext(Component component, FontMetrics fontMetrics) {
        assert component != null;
        return (fontMetrics != null) ? fontMetrics.getFontRenderContext() : getFontRenderContext(component);
    }
    
    public static FontMetrics getFontMetrics(JComponent component, Font font) {
        FontRenderContext frc = getFRCProperty(component);
        if (frc == null) {
            frc = SwingUtilities2.DEFAULT_FRC;
        }
        return FontDesignMetrics.getMetrics(font, frc);
    }
    
    private static FontRenderContext getFRCProperty(JComponent component) {
        if (component != null) {
            SwingUtilities2.AATextInfo aaTextInfo = (SwingUtilities2.AATextInfo)component.getClientProperty(SwingUtilities2.AA_TEXT_PROPERTY_KEY);
            if (aaTextInfo != null) {
                return aaTextInfo.frc;
            }
        }
        return null;
    }
    
    static boolean isPrinting(Graphics graphics) {
        return graphics instanceof PrinterGraphics || graphics instanceof PrintGraphics;
    }
    
    private static String trimTrailingSpaces(String s) {
        int index;
        for (index = s.length() - 1; index >= 0 && Character.isWhitespace(s.charAt(index)); --index) {}
        return s.substring(0, index + 1);
    }
    
    private static AttributedCharacterIterator getTrimmedTrailingSpacesIterator(AttributedCharacterIterator text) {
        int index = text.getIndex();
        int ch;
        for (ch = text.last(); ch != '\uffff' && Character.isWhitespace((char)ch); ch = text.previous()) {}
        if (ch != '\uffff') {
            int index2 = text.getIndex();
            if (index2 == text.getEndIndex() - 1) {
                text.setIndex(index);
                return text;
            }
            return new AttributedString(text, text.getBeginIndex(), index2 + 1).getIterator();
        }
        else {
            return null;
        }
    }
    
    public static boolean useSelectedTextColor(Highlighter.Highlight highlight, JTextComponent textComponent) {
        Highlighter.HighlightPainter painter = highlight.getPainter();
        String name = painter.getClass().getName();
        if (name.indexOf("javax.swing.text.DefaultHighlighter") != 0 && name.indexOf("com.sun.java.swing.plaf.windows.WindowsTextUI") != 0) {
            return false;
        }
        try {
            DefaultHighlighter.DefaultHighlightPainter defaultHighlightPainter = (DefaultHighlighter.DefaultHighlightPainter)painter;
            if (defaultHighlightPainter.getColor() != null && !defaultHighlightPainter.getColor().equals(textComponent.getSelectionColor())) {
                return false;
            }
        }
        catch (ClassCastException ex) {
            return false;
        }
        return true;
    }
    
    public static boolean canAccessSystemClipboard() {
        boolean canCurrentEventAccessSystemClipboard = false;
        if (!GraphicsEnvironment.isHeadless()) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager == null) {
                canCurrentEventAccessSystemClipboard = true;
            }
            else {
                try {
                    securityManager.checkPermission(SecurityConstants.AWT.ACCESS_CLIPBOARD_PERMISSION);
                    canCurrentEventAccessSystemClipboard = true;
                }
                catch (SecurityException ex) {}
                if (canCurrentEventAccessSystemClipboard && !isTrustedContext()) {
                    canCurrentEventAccessSystemClipboard = canCurrentEventAccessSystemClipboard(true);
                }
            }
        }
        return canCurrentEventAccessSystemClipboard;
    }
    
    public static boolean canCurrentEventAccessSystemClipboard() {
        return isTrustedContext() || canCurrentEventAccessSystemClipboard(false);
    }
    
    public static boolean canEventAccessSystemClipboard(AWTEvent awtEvent) {
        return isTrustedContext() || canEventAccessSystemClipboard(awtEvent, false);
    }
    
    private static synchronized boolean inputEvent_canAccessSystemClipboard(InputEvent obj) {
        if (SwingUtilities2.inputEvent_CanAccessSystemClipboard_Field == null) {
            SwingUtilities2.inputEvent_CanAccessSystemClipboard_Field = (Field)AccessController.doPrivileged(new PrivilegedAction<Field>() {
                @Override
                public Field run() {
                    try {
                        Field declaredField = InputEvent.class.getDeclaredField("canAccessSystemClipboard");
                        declaredField.setAccessible(true);
                        return declaredField;
                    }
                    catch (SecurityException ex) {}
                    catch (NoSuchFieldException ex2) {}
                    return null;
                }
            });
        }
        if (SwingUtilities2.inputEvent_CanAccessSystemClipboard_Field == null) {
            return false;
        }
        boolean boolean1 = false;
        try {
            boolean1 = SwingUtilities2.inputEvent_CanAccessSystemClipboard_Field.getBoolean(obj);
        }
        catch (IllegalAccessException ex) {}
        return boolean1;
    }
    
    private static boolean isAccessClipboardGesture(InputEvent inputEvent) {
        boolean n = false;
        if ((inputEvent instanceof KeyEvent)) {
            KeyEvent keyEvent = (KeyEvent)inputEvent;
            int keyCode = keyEvent.getKeyCode();
            int modifiers = keyEvent.getModifiers();
            switch (keyCode) {
                case 67:
                case 86:
                case 88: {
                    n = (modifiers == 2);
                    break;
                }
                case 155: {
                    n = (modifiers == 2 || modifiers == 1);
                    break;
                }
                case 65485:
                case 65487:
                case 65489: {
                    n = true;
                    break;
                }
                case 127: {
                    n = (modifiers == 1);
                    break;
                }
            }
        }
        return n;
    }
    
    private static boolean canEventAccessSystemClipboard(AWTEvent awtEvent, boolean b) {
        return !EventQueue.isDispatchThread() || (awtEvent instanceof InputEvent && (!b || isAccessClipboardGesture((InputEvent)awtEvent)) && inputEvent_canAccessSystemClipboard((InputEvent)awtEvent));
    }
    
    public static void checkAccess(int mod) {
        if (System.getSecurityManager() != null && !Modifier.isPublic(mod)) {
            throw new SecurityException("Resource is not accessible");
        }
        return;
    }
    
    private static boolean canCurrentEventAccessSystemClipboard(boolean b) {
        return canEventAccessSystemClipboard(EventQueue.getCurrentEvent(), b);
    }
    
    private static boolean isTrustedContext() {
        return System.getSecurityManager() == null || AppContext.getAppContext().get("UNTRUSTED_CLIPBOARD_ACCESS_KEY") == null;
    }
    
    public static String displayPropertiesToCSS(Font font, Color color) {
        StringBuffer sb = new StringBuffer("body {");
        if (font != null) {
            sb.append(" font-family: ");
            sb.append(font.getFamily());
            sb.append(" ; ");
            sb.append(" font-size: ");
            sb.append(font.getSize());
            sb.append("pt ;");
            if (font.isBold()) {
                sb.append(" font-weight: 700 ; ");
            }
            if (font.isItalic()) {
                sb.append(" font-style: italic ; ");
            }
        }
        if (color != null) {
            sb.append(" color: #");
            if (color.getRed() < 16) {
                sb.append('0');
            }
            sb.append(Integer.toHexString(color.getRed()));
            if (color.getGreen() < 16) {
                sb.append('0');
            }
            sb.append(Integer.toHexString(color.getGreen()));
            if (color.getBlue() < 16) {
                sb.append('0');
            }
            sb.append(Integer.toHexString(color.getBlue()));
            sb.append(" ; ");
        }
        sb.append(" }");
        return sb.toString();
    }
    
    public static Object makeIcon(final Class<?> clazz, final Class<?> clazz2, final String s) {
        return new UIDefaults.LazyValue() {
            @Override
            public Object createValue(UIDefaults uiDefaults) {
                byte[] array = (byte[])AccessController.doPrivileged(new PrivilegedAction<byte[]>() {
                    @Override
                    public byte[] run() {
                        try {
                            InputStream resourceAsStream = null;
                            Class<Object> clazz = clazz;
                            while (clazz != null) {
                                resourceAsStream = clazz.getResourceAsStream(s);
                                if (resourceAsStream == null) {
                                    if (clazz == clazz2) {
                                        break;
                                    }
                                    clazz = clazz.getSuperclass();
                                }
                                else {
                                    break;
                                }
                            }
                            if (resourceAsStream == null) {
                                return null;
                            }
                            BufferedInputStream o = new BufferedInputStream(resourceAsStream);
                            ByteArrayOutputStream o2 = new ByteArrayOutputStream(1024);
                            byte[] array = new byte[1024];
                            int read;
                            while ((read = o.read(array)) > 0) {
                                o2.write(array, 0, read);
                            }
                            o.close();
                            o2.flush();
                            return o2.toByteArray();
                        }
                        catch (IOException ex) {
                            System.err.println(ex.toString());
                            return null;
                        }
                    }
                });
                if (array == null) {
                    return null;
                }
                if (array.length == 0) {
                    System.err.println("warning: " + s + " is zero-length");
                    return null;
                }
                return new ImageIconUIResource(array);
            }
        };
    }
    
    public static boolean isLocalDisplay() {
        GraphicsEnvironment localGraphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
        return !(localGraphicsEnvironment instanceof SunGraphicsEnvironment) || ((SunGraphicsEnvironment)localGraphicsEnvironment).isDisplayLocal();
    }
    
    public static int getUIDefaultsInt(Object o) {
        return getUIDefaultsInt(o, 0);
    }
    
    public static int getUIDefaultsInt(Object o, Locale locale) {
        return getUIDefaultsInt(o, locale, 0);
    }
    
    public static int getUIDefaultsInt(Object o, int n) {
        return getUIDefaultsInt(o, null, n);
    }
    
    public static int getUIDefaultsInt(Object key, Locale l, int n) {
        Object value = UIManager.get(key, l);
        if ((value instanceof Integer)) {
            return (int)value;
        }
        if ((value instanceof String)) {
            try {
                return Integer.parseInt((String)value);
            }
            catch (NumberFormatException ex) {}
        }
        return n;
    }
    
    public static Component compositeRequestFocus(Component component) {
        if ((component instanceof Container)) {
            Container b = (Container)component;
            if (b.isFocusCycleRoot()) {
                Component defaultComponent = b.getFocusTraversalPolicy().getDefaultComponent(b);
                if (defaultComponent != null) {
                    defaultComponent.requestFocus();
                    return defaultComponent;
                }
            }
            Container focusCycleRootAncestor = b.getFocusCycleRootAncestor();
            if (focusCycleRootAncestor != null) {
                Component componentAfter = focusCycleRootAncestor.getFocusTraversalPolicy().getComponentAfter(focusCycleRootAncestor, b);
                if (componentAfter != null && SwingUtilities.isDescendingFrom(componentAfter, b)) {
                    componentAfter.requestFocus();
                    return componentAfter;
                }
            }
        }
        if (component.isFocusable()) {
            component.requestFocus();
            return component;
        }
        return null;
    }
    
    public static boolean tabbedPaneChangeFocusTo(Component component) {
        if (component != null) {
            if (component.isFocusTraversable()) {
                compositeRequestFocus(component);
                return true;
            }
            if (component instanceof JComponent && ((JComponent)component).requestDefaultFocus()) {
                return true;
            }
        }
        return false;
    }
    
    public static <V> Future<V> submit(Callable<V> callable) {
        if (callable == null) {
            throw new NullPointerException();
        }
        FutureTask futureTask = new FutureTask(callable);
        execute(futureTask);
        return futureTask;
    }
    
    public static <V> Future<V> submit(Runnable runnable, V result) {
        if (runnable == null) {
            throw new NullPointerException();
        }
        FutureTask futureTask = new FutureTask(runnable, result);
        execute(futureTask);
        return futureTask;
    }
    
    private static void execute(Runnable doRun) {
        SwingUtilities.invokeLater(doRun);
    }
    
    public static void setSkipClickCount(Component component, int i) {
        if (component instanceof JTextComponent && ((JTextComponent)component).getCaret() instanceof DefaultCaret) {
            ((JTextComponent)component).putClientProperty(SwingUtilities2.SKIP_CLICK_COUNT, Integer.valueOf(i));
        }
    }
    
    public static int getAdjustedClickCount(JTextComponent textComponent, MouseEvent mouseEvent) {
        int clickCount = mouseEvent.getClickCount();
        if (clickCount == 1) {
            textComponent.putClientProperty(SwingUtilities2.SKIP_CLICK_COUNT, null);
        }
        else {
            Integer n = (Integer)textComponent.getClientProperty(SwingUtilities2.SKIP_CLICK_COUNT);
            if (n != null) {
                return clickCount - (int)n;
            }
        }
        return clickCount;
    }
    
    private static SwingUtilities2.Section liesIn(Rectangle rectangle, Point point, boolean b, boolean n, boolean b2) {
        int n2;
        int n3;
        int n4;
        if (b) {
            n2 = rectangle.x;
            n3 = point.x;
            n4 = rectangle.width;
        }
        else {
            n2 = rectangle.y;
            n3 = point.y;
            n4 = rectangle.height;
            n = 1;
        }
        if (b2) {
            int n6 = (n4 >= 30) ? 10 : (n4 / 3);
            if (n3 < n2 + n6) {
                return (n != 0) ? SwingUtilities2.Section.LEADING : SwingUtilities2.Section.TRAILING;
            }
            if (n3 >= n2 + n4 - n6) {
                return (n != 0) ? SwingUtilities2.Section.TRAILING : SwingUtilities2.Section.LEADING;
            }
            return SwingUtilities2.Section.MIDDLE;
        }
        else {
            int n5 = n2 + n4 / 2;
            if (n != 0) {
                return (n3 >= n5) ? SwingUtilities2.Section.TRAILING : SwingUtilities2.Section.LEADING;
            }
            return (n3 < n5) ? SwingUtilities2.Section.TRAILING : SwingUtilities2.Section.LEADING;
        }
    }
    
    public static SwingUtilities2.Section liesInHorizontal(Rectangle rectangle, Point point, boolean b, boolean b2) {
        return liesIn(rectangle, point, true, b, b2);
    }
    
    public static SwingUtilities2.Section liesInVertical(Rectangle rectangle, Point point, boolean b) {
        return liesIn(rectangle, point, false, false, b);
    }
    
    public static int convertColumnIndexToModel(TableColumnModel tableColumnModel, int n) {
        if (n < 0) {
            return n;
        }
        return tableColumnModel.getColumn(n).getModelIndex();
    }
    
    public static int convertColumnIndexToView(TableColumnModel tableColumnModel, int n) {
        if (n < 0) {
            return n;
        }
        int i = 0;
        while (i < tableColumnModel.getColumnCount()) {
            if (tableColumnModel.getColumn(i).getModelIndex() == n) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    public static int getSystemMnemonicKeyMask() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        if ((defaultToolkit instanceof SunToolkit)) {
            return ((SunToolkit)defaultToolkit).getFocusAcceleratorKeyMask();
        }
        return 8;
    }
    
    public static TreePath getTreePath(TreeModelEvent treeModelEvent, TreeModel treeModel) {
        TreePath treePath = treeModelEvent.getTreePath();
        if (treePath == null && treeModel != null) {
            Object root = treeModel.getRoot();
            if (root != null) {
                treePath = new TreePath(root);
            }
        }
        return treePath;
    }
    
    static {
        LAF_STATE_KEY = new StringBuffer("LookAndFeel State");
        MENU_SELECTION_MANAGER_LISTENER_KEY = new StringBuffer("MenuSelectionManager listener key");
        DEFAULT_FRC = new FontRenderContext(null, false, false);
        AA_TEXT_PROPERTY_KEY = new StringBuffer("AATextInfoPropertyKey");
        SKIP_CLICK_COUNT = new StringBuilder("skipClickCount");
        COMPONENT_UI_PROPERTY_KEY = new StringBuffer("ComponentUIPropertyKey");
        BASICMENUITEMUI_MAX_TEXT_OFFSET = new StringUIClientPropertyKey("maxTextOffset");
        SwingUtilities2.inputEvent_CanAccessSystemClipboard_Field = null;
        charsBufferLock = new Object();
        SwingUtilities2.charsBuffer = new char[100];
        SwingUtilities2.fontCache = new SwingUtilities2.LSBCacheEntry[6];
    }
    
    public static class AATextInfo
    {
        Object aaHint;
        Integer lcdContrastHint;
        FontRenderContext frc;
        
        private static SwingUtilities2.AATextInfo getAATextInfoFromMap(Map map) {
            Object value = map.get(RenderingHints.KEY_TEXT_ANTIALIASING);
            Object value2 = map.get(RenderingHints.KEY_TEXT_LCD_CONTRAST);
            if (value == null || value == RenderingHints.VALUE_TEXT_ANTIALIAS_OFF || value == RenderingHints.VALUE_TEXT_ANTIALIAS_DEFAULT) {
                return null;
            }
            return new SwingUtilities2.AATextInfo(value, (Integer)value2);
        }
        
        public static SwingUtilities2.AATextInfo getAATextInfo(boolean aaFontSettingsCondition) {
            SunToolkit.setAAFontSettingsCondition(aaFontSettingsCondition);
            Object desktopProperty = Toolkit.getDefaultToolkit().getDesktopProperty("awt.font.desktophints");
            if ((desktopProperty instanceof Map)) {
                return getAATextInfoFromMap((Map)desktopProperty);
            }
            return null;
        }
        
        public AATextInfo(Object o, Integer lcdContrastHint) {
            if (o == null) {
                throw new InternalError("null not allowed here");
            }
            if (o == RenderingHints.VALUE_TEXT_ANTIALIAS_OFF || o == RenderingHints.VALUE_TEXT_ANTIALIAS_DEFAULT) {
                throw new InternalError("AA must be on");
            }
            this.aaHint = o;
            this.lcdContrastHint = lcdContrastHint;
            this.frc = new FontRenderContext(null, o, RenderingHints.VALUE_FRACTIONALMETRICS_DEFAULT);
            return;
        }
    }
    
    private static class LSBCacheEntry
    {
        private static final byte UNSET = Byte.MAX_VALUE;
        private static final char[] oneChar;
        private byte[] lsbCache;
        private Font font;
        private FontRenderContext frc;
        
        public LSBCacheEntry(FontRenderContext fontRenderContext, Font font) {
            this.lsbCache = new byte[1];
            this.reset(fontRenderContext, font);
        }
        
        public void reset(FontRenderContext frc, Font font) {
            this.font = font;
            this.frc = frc;
            for (int i = this.lsbCache.length - 1; i >= 0; --i) {
                this.lsbCache[i] = 127;
            }
        }
        
        public int getLeftSideBearing(char c) {
            int n = c - 87;
            assert n >= 0 && n < 1;
            int n2 = this.lsbCache[n];
            if (n2 == 127) {
                SwingUtilities2.LSBCacheEntry.oneChar[0] = c;
                n2 = (byte)this.font.createGlyphVector(this.frc, SwingUtilities2.LSBCacheEntry.oneChar).getGlyphPixelBounds(0, this.frc, 0.0f, 0.0f).x;
                if (n2 < 0) {
                    Object antiAliasingHint = this.frc.getAntiAliasingHint();
                    if (antiAliasingHint == RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB || antiAliasingHint == RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HBGR) {
                        n2 = (byte)(n2 + 1);
                    }
                }
                this.lsbCache[n] = n2;
            }
            return n2;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if (!(o instanceof SwingUtilities2.LSBCacheEntry)) {
                return false;
            }
            SwingUtilities2.LSBCacheEntry lsbCacheEntry = (SwingUtilities2.LSBCacheEntry)o;
            return this.font.equals(lsbCacheEntry.font) && this.frc.equals(lsbCacheEntry.frc);
        }
        
        @Override
        public int hashCode() {
            int n = 17;
            if (this.font != null) {
                n = 37 * n + this.font.hashCode();
            }
            if (this.frc != null) {
                n = 37 * n + this.frc.hashCode();
            }
            return n;
        }
        
        static {
            oneChar = new char[1];
        }
    }
    
    public enum Section
    {
        LEADING, 
        MIDDLE, 
        TRAILING;
    }
    
    public interface RepaintListener
    {
        public abstract void repaintPerformed(JComponent p0, int p1, int p2, int p3, int p4);
    }
}
